# OZIrectorPile.rb  http://www.est.hi-ho.ne.jp/atamadekkati/ 
# ダイアログボックスを表示し、入力した長さのパイプを指定点に描画する。
# 2009/ 8/23 ver0.50 イレクター用に作成
# 2009/10/31 ver1.00 場合によって、パイプが逆方向に延びるのを修正。
#
# 使用者の自己責任でご使用下さい。
# 改変してマシなのができたら私にも教えてください。
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

class OZIrectorPipe

def initialize
    @ip1 = nil
    @ip = nil
    @valuedata = [500.mm, "Ｘ軸", "28mm", "" ]
end

# The activate method is called by SketchUp when the tool is first selected.
def activate
  @ip1 = Sketchup::InputPoint.new
  @ip = Sketchup::InputPoint.new
  @drawn = false

  self.reset(nil)
end

# Reset the tool back to its initial state
def reset(view)
    self.promptmake
    # Display a prompt on the status bar
    Sketchup::set_status_text("配置する座標をクリックしてください。", SB_PROMPT)
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
end

# 入力欄が毎回でるようにactivateから分ける。
def promptmake
  prompts = ["パイプの長さ", "作成する軸方向", "パイプの種類", "名前"]
  enums=[nil, "Ｘ軸|Ｙ軸|Ｚ軸", "28mm|42mm", nil]
  values = @valuedata
  results = inputbox prompts, values, enums, "入力"
  return if not results # This means that the user canceld the operation
  @valuedata = results
  @pipelength = @valuedata[0]
  if @valuedata[1] == "Ｘ軸" then
    @pipevec = Geom::Vector3d.new 1,0,0
  elsif @valuedata[1] == "Ｙ軸" then
    @pipevec = Geom::Vector3d.new 0,1,0
  else    # Ｚ軸
    @pipevec = Geom::Vector3d.new 0,0,1
  end
  if @valuedata[2] == "28mm" then
    @pipekei = 28.mm
    @pipeatu = 1.mm
    @guidept = 3.mm
    @flatsw = 0
  elsif @valuedata[2] == "42mm" then
    @pipekei = 42.mm
    @pipeatu = 1.mm
    @guidept = 3.mm
    @flatsw = 0
  elsif @valuedata[2] == "34mmプラ" then
    @pipekei = 34.mm
    @pipeatu = 3.mm
    @guidept = 0.mm
    @flatsw = 0
  elsif @valuedata[2] == "フラット" then   # フラットはまだ対応していない。
    @pipekei = 28.mm
    @flatsw = 1
  end
  @pipename = @valuedata[3]
end

# deactivate is called when the tool is deactivated because a different tool was selected
def deactivate(view)
  view.invalidate if @drawn
end

# The onMouseMove method is called whenever the user moves the mouse.
def onMouseMove(flags, x, y, view)
  @ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
    view.tooltip = @ip1.tooltip
  end
end

# The onLButtonDOwn method is called when the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
  if( @ip1.valid? )
    self.create_irepipe(@ip1.position, view)
    self.reset(view)
  end
  view.lock_inference
end

# onKeyDown is called when the user presses a key on the keyboard.
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @ip1.valid? )
            view.lock_inference @ip1
        end
    end
end

# onKeyUp is called when the user releases the key
def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end

# Create new geometry when the user has selected point.
def create_irepipe(point1,  view)
    model = Sketchup.active_model
    model.start_operation "パイプを作る"
    entities = model.active_entities
    group = entities.add_group
    entities = group.entities
    group.name = @pipename.to_s

    centervec = Geom::Vector3d.new 1,1,1
    centervec = centervec - @pipevec
    #円筒
    shiftkei = @pipekei / 2 + @guidept
    centerpts = point1 + [ centervec[0] * shiftkei, centervec[1] * shiftkei, centervec[2] * shiftkei ]  
    outerbase = entities.add_circle centerpts, @pipevec, @pipekei / 2
    facebase = entities.add_face outerbase
    innerbase =  entities.add_circle centerpts, @pipevec, @pipekei / 2 - @pipeatu
    innerbase[0].faces.each {|f| f.erase! if( f != facebase)}

       #プッシュプルする方向を決める。
    if (@pipevec[0] > 0.5 and facebase.normal.dot(X_AXIS) < 0.0) then
      pushlength = -@pipelength
    elsif (@pipevec[1] > 0.5 and facebase.normal.dot(Y_AXIS) < 0.0) then
      pushlength = -@pipelength
    elsif (@pipevec[2] > 0.5 and facebase.normal.dot(Z_AXIS) < 0.0) then
      pushlength = -@pipelength
    else
      pushlength = @pipelength
    end

    facebase.pushpull pushlength

    #中心線 を描くとパイプ長さを変更しにくくなるので、描画しない。
    transleng = Geom::Transformation.translation [ @pipevec[0] * @pipelength, @pipevec[1] * @pipelength, @pipevec[2] * @pipelength ]
#    centerpts2 = transleng * centerpts 
#    centerline = entities.add_line centerpts, centerpts2

    #ガイドポイント 
    trans90 = Geom::Transformation.rotation centerpts, @pipevec, Math::PI / 2
    constpoint = entities.add_cpoint point1
    tranpoint = trans90 * point1
    constpoint = entities.add_cpoint tranpoint
    tranpoint = trans90 * tranpoint
    constpoint = entities.add_cpoint tranpoint
    tranpoint = trans90 * tranpoint
    constpoint = entities.add_cpoint tranpoint
    tranpoint = transleng * point1
    constpoint = entities.add_cpoint tranpoint
    tranpoint = trans90 * tranpoint
    constpoint = entities.add_cpoint tranpoint
    tranpoint = trans90 * tranpoint
    constpoint = entities.add_cpoint tranpoint
    tranpoint = trans90 * tranpoint
    constpoint = entities.add_cpoint tranpoint

    model.commit_operation
end

end # class 

#=============================================================================
def irector_pipe
    Sketchup.active_model.select_tool OZIrectorPipe.new
end

# Add a menu 
if( not file_loaded?("OZIrectorPipe.rb") )
    add_separator_to_menu("Draw")
    UI.menu("Draw").add_item("イレクターパイプ") { irector_pipe }
end

#-----------------------------------------------------------------------------
file_loaded("OZIrectorPipe.rb")
